@@ -17,33 +17,46 @@ import random
 import re
 import csv
 
-try:
-    # Display logo on Pi-Top[4]
-    from ptoled import PTOLEDDisplay
-    oled_screen = PTOLEDDisplay()
-    oled_screen.draw_image_file("Screen/gif.gif")
-except:
-    pass
+sapivoice = ""
+sapistream = ""
+
 # Import the correct sound library depending on platform.
 if platform.system() == "Windows":
     import winsound
     # For SAPI speech
     from comtypes.client import CreateObject
+    sapivoice = CreateObject("SAPI.SpVoice")
+    sapistream = CreateObject("SAPI.SpFileStream")
+    from comtypes.gen import SpeechLib
 if platform.system() == "Darwin":
     from playsound import playsound
 if platform.system() == "Linux":
     from playsound import playsound
     from gtts import gTTS
     from pydub import AudioSegment
+  
+# Variables to hold name of settings
 
-global writing, language, voice, synthesizer, speechRate, connected, shapeList, phraseList, topLipFree, directory
+speechDatabaseFile = ''
+defaultEyeShape = ''
+eyeShapeFile = ''
+speechAudioFile = 'picohData/picohspeech.wav'
+picohMotorDefFile = 'picohData/MotorDefinitonsPicoh.omd'
+soundFolder = 'picohData/Sounds'
+synthesizer = ''
+voice = ''
+language = 'en-GB' # Language/Accent for GTTS web text to speech.
+settingsFile = 'picohData/PicohSettings.xml' # String to hold location of settings file
+
+# Variable to hold the location of the picoh library folder.
+directory = os.path.dirname(os.path.abspath(__file__))
+
+# name of directory that holds eye shape, motor definition and speech db files.
+dirName = 'picohData'
 
 # Global variable to turn on and off printing debug info.
 debug = False
 
-# Language in which you want to convert used in GTTS web text to speech.
-language = 'en-us'
-
 # define constants for motors
 HEADNOD = 0
 HEADTURN = 1
@@ -73,7 +86,7 @@ phraseList = []
 port = ""
 
 # define library version
-version = "1.0"
+version = "1.255"
 
 # flag to stop writing when writing for threading
 writing = False
@@ -84,31 +97,6 @@ connected = False
 # flag to track if top lip is below centre.
 topLipFree = False
 
-# global to set the params to speech synthesizer which control the voice
-if platform.system() == "Windows":
-    voice = ""
-if platform.system() == "Darwin":
-    voice = ""
-if platform.system() == "Linux":
-    voice = ""
-
-# global to set the speech speed.
-speechRate = 170
-
-# Global flag to use a synthesizer other than sapi.
-# If it's not sapi then it needs to support -w parameter to write to file e.g. espeak or espeak-NG
-if platform.system() == "Windows":
-    synthesizer = "sapi"
-if platform.system() == "Darwin":
-    synthesizer = "say -o "
-if platform.system() == "Linux":
-    synthesizer = "festival"
-
-print("Speech Synthesizer: " + synthesizer)
-
-# name of directory that holds eye shape, motor definition and speech db files.
-dirName = 'picohData'
-
 # If the picohData folder does not exist, create it.
 try:
     # Create target Directory
@@ -116,41 +104,105 @@ try:
 except FileExistsError:
     pass
 
-# Variable to hold the location of the picoh library folder.
-directory = os.path.dirname(os.path.abspath(__file__))
+# Copy settings file
+if not path.exists('picohData/PicohSettings.xml'):
+    shutil.copyfile(os.path.join(directory, "PicohSettings.xml"), settingsFile)
+    if debug:
+        print("Copied Picoh Settings from :" + directory + " to picohData/")
 
-if debug:
-    print("Picoh Library Directory " + directory)
+# Load settings from XML file.
+def _loadSettings():
+    global eyeShapeFile,defaultEyeShape,synthesizer,voice,speechDatabaseFile,eyeShapeFile,picohMotorDefFile
+    
+    tree = etree.parse(settingsFile)
 
-if not path.exists('picohData/PicohSpeech.csv'):
-    shutil.copyfile(os.path.join(directory, 'PicohSpeech.csv'), 'picohData/PicohSpeech.csv')
+    
+    root = tree.getroot()
+
+    for element in tree.iter():
+
+        name = element.get("Name")
+        val = element.get("Value")
+
+        if name == "DefaultEyeShape":
+            defaultEyeShape = val
+            
+        if name == "DefaultSpeechSynth":
+            synthesizer = val
+
+        if name == "DefaultVoice":
+            voice = val
+
+        if name == "DefaultLang":
+            language = val
+
+        if name == "SpeechDBFile":
+            speechDatabaseFile = val
+
+        if name == "EyeShapeList":
+            eyeShapeFile = val
+
+        if name == "MotorDefFile":
+            picohMotorDefFile = val
+
+_loadSettings()
+
+# Copy over files
+if not path.exists(speechDatabaseFile):
+    shutil.copyfile(os.path.join(directory, 'PicohSpeech.csv'),speechDatabaseFile)
     if debug:
         print("Copied PicohSpeech.csv from :" + directory + " to picohData/")
 
-if not path.exists('picohData/ohbot.obe'):
-    shutil.copyfile(os.path.join(directory, 'ohbot.obe'), 'picohData/ohbot.obe')
+if not path.exists(eyeShapeFile):
+    shutil.copyfile(os.path.join(directory, 'ohbot.obe'), eyeShapeFile)
     if debug:
         print("Copied ohbot.obe from :" + directory + " to picohData/")
 
-if not path.exists('picohData/MotorDefinitionsPicoh.omd'):
-    shutil.copyfile(os.path.join(directory, 'MotorDefinitionsPicoh.omd'), 'picohData/MotorDefinitionsPicoh.omd')
+if not path.exists(picohMotorDefFile):
+    shutil.copyfile(os.path.join(directory, 'MotorDefinitionsPicoh.omd'), picohMotorDefFile)
     if debug:
         print("Copied MotorDefinitionsPicoh.omd from :" + directory + " to picohData/")
 
-if not path.exists('picohData/Sounds'):
+if not path.exists(soundFolder):
 
-    shutil.copytree(os.path.join(directory, 'Sounds'), 'picohData/Sounds')
+    shutil.copytree(os.path.join(directory, 'Sounds'),soundFolder)
 
     if debug:
         print("Copied Sounds from :" + directory + " to PicohData/")
 
-# Variables to hold name of speech database and eyeshape files.
+# global to set the params to speech synthesizer which control the voice
+# Global flag to use a synthesizer other than sapi.
+# If it's not sapi then it needs to support -w parameter to write to file e.g. espeak or espeak-NG
+
+if platform.system() == "Windows":
+    if synthesizer == "":
+        synthesizer = "sapi"
+if platform.system() == "Darwin":
+    if synthesizer == "":
+        synthesizer = "say -o "
+if platform.system() == "Linux":
+    if synthesizer == "":
+        synthesizer = "festival"
+    # un-comment to make google webspeech default
+    # synthesizer = "gTTS" 
+
+# global to set the speech speed.
+speechRate = 170
+
+print("Speech Synthesizer: " + synthesizer)
+
+try:
+    # Display logo on Pi-Top[4]
+    from ptoled import PTOLEDDisplay
+    oled_screen = PTOLEDDisplay()
+    logoImagePath = os.path.join(directory, 'Images/logoPT.gif')
+    oled_screen.draw_image_file(logoImagePath)
+except:
+    pass
+
+if debug:
+    print("Picoh Library Directory " + directory)
 
-speechDatabaseFile = 'picohData/PicohSpeech.csv'
-eyeShapeFile = 'picohData/ohbot.obe'
-speechAudioFile = 'picohData/picohspeech.wav'
-picohMotorDefFile = 'picohData/MotorDefinitionsPicoh.omd'
-soundFolder = 'picohData/Sounds'
 
 # Cache of pupil positions
 global lastfex, lastfey
@@ -178,10 +230,10 @@ class Phrase(object):
         self.set = set
         self.variable = variable
         self.text = text
-
+            
 # Used during calibration.
 def _revertMotorDefsFile():
-    shutil.copyfile(os.path.join(directory, 'MotorDefinitionsPicoh.omd'), 'picohData/MotorDefinitionsPicoh.omd')
+    shutil.copyfile(os.path.join(directory, 'MotorDefinitionsPicoh.omd'), picohMotorDefFile)
     _loadMotorDefs()
 
 # Read motor definitions file.
@@ -205,18 +257,14 @@ def _loadMotorDefs():
             rev = False
             motorRev[index] = rev
 
-
+            
 # Read eyeshape file into eyeshape list.
 def _loadEyeShapes():
     global shapeList
 
     # Clear the shapeList
     shapeList = []
-
-    file = eyeShapeFile
-
-    tree = etree.parse(file)
-
+    tree = etree.parse(eyeShapeFile)
     index = 0
 
     for element in tree.iter():
@@ -241,8 +289,7 @@ def _loadEyeShapes():
                 shapeList[index].autoMirror = False
 
             index = index + 1
-
-
+       
 # Read speech database file into phraseList.
 def _loadSpeechDatabase():
     global phraseList
@@ -274,7 +321,7 @@ def _loadSpeechDatabase():
                     newPhrase = Phrase(row[0], row[1], row[2])
                     phraseList.append(newPhrase)
 
-
+                    
 # Function to check if a number is a digit including negative numbers
 def _is_digit(n):
     try:
@@ -299,16 +346,14 @@ def _parseSAPIVoice(flag):
             val = val[:pos]
     return val
 
-
-# speak depending on synthesizer
-def _speak(text):
+# generate speech file depending on platform and sythensizer.
+def _generateSpeechFile(text):
     # Pick up the global variable that defines the language. This is only used in GTTS speech.
-
     global language
     file = speechAudioFile
     if platform.system() == "Windows":
         if ("sapi" in synthesizer.lower()):
-            from comtypes.gen import SpeechLib
+            #from comtypes.gen import SpeechLib
             global sapivoice, sapistream
 
             sapistream.Open(file, SpeechLib.SSFMCreateForWrite)
@@ -379,37 +424,35 @@ def _speak(text):
 
             # Saving the converted audio in a wav file named sample
             tts.save('picohData/picohspeech.mp3')
-                        # Saving the converted audio in a wav file named sample
-            #tts.save('picohData/picohspeech.mp3')
-            #print("gtts")
             sound = AudioSegment.from_mp3('picohData/picohspeech.mp3')
             sound.export('picohData/picohspeech.wav', format="wav")
         else:
             # Remove any characters that are unsafe for a subprocess call
             safetext = re.sub(r'[^ .a-zA-Z0-9?\']+', '', text)
-            bashcommand = synthesizer + ' -w picohData/picohspeech.wav ' + voice + ' "' + safetext + '"'
+                      
+            if synthesizer.upper() == "FESTIVAL":
+                bashcommand = "festival -b '(set! mytext (Utterance Text " + '"' + safetext + '"))' + "' '(utt.synth mytext)' '(utt.save.wave mytext " + '"picohData/picohspeech.wav")' + "' '(utt.save.segs mytext " + '"picohData/phonemes"' + ")'"
+            else:
+                bashcommand = synthesizer + ' -w picohData/picohspeech.wav ' + voice + ' "' + safetext + '"'
+                
             # Execute bash command.
             subprocess.call(bashcommand, shell=True)
 
-        if debug:
-            print("Speech Bash Command:")
-            print(bashcommand)
-
+            if debug:
+                print("Speech Bash Command:")
+                print(bashcommand)
 
 def init(portName):
     # pickup global instances of port, ser and sapi variables
-    global port, ser, sapivoice, sapistream, connected
+    global port, ser, sapivoice, sapistream, connected, directory
 
     _loadEyeShapes()
     _loadMotorDefs()
 
-    dir = os.path.dirname(os.path.abspath(__file__))
-    silenceFile = os.path.join(dir, 'Silence1.wav')
+    silenceFile = os.path.join(directory, 'Silence1.wav')
 
     # get the sapi objects ready on Windows
     if platform.system() == "Windows":
-        sapivoice = CreateObject("SAPI.SpVoice")
-        sapistream = CreateObject("SAPI.SpFileStream")
         winsound.PlaySound(silenceFile, winsound.SND_FILENAME)
 
     # get the audio system warmed up on Mac
@@ -460,9 +503,9 @@ def init(portName):
     # Create a bash command with the desired text. The command writes two files, a .wav with the speech audio and a
     # .txt file containing the phonemes and the times.
 
-    if synthesizer != "festival":
-        _speak(text)
-
+    if synthesizer.lower() != "festival":
+        _generateSpeechFile(text)
+        
     _loadSpeechDatabase()
 
     return True
@@ -476,12 +519,10 @@ if platform.system() == "Darwin":
 if platform.system() == "Linux":
     init("Arduino")
 
-
 def getDirectory():
     global directory
     return directory
 
-
 # Function to move Picoh's motors. Arguments | m (motor) → int (0-6) | pos (position) → int (0-10) | spd (speed) →
 # int (0-10) **eg move(4,3,9) or move(0,9,3)**
 def move(m, pos, spd=5, eye=0):
@@ -512,8 +553,7 @@ def move(m, pos, spd=5, eye=0):
     # Eyeturn
     if (motorType[m] == "Matrix X"):
         pos = (pos - 5) * 0.4 + 5  # Bodge to reduce the scale
-        msg = "FE," + str(eye) + "," + "{:0.0f}".format(pos * 255 / 10) + "," + "{:0.0f}".format(
-            lastfey * 255 / 10) + "\n"
+        msg = "FE," + str(eye) + "," + "{:0.0f}".format(pos * 255 / 10) + "," + "{:0.0f}".format(lastfey * 255 / 10) + "\n"
 
         # print ("fex:" + msg)
         # Write message to serial port
@@ -524,8 +564,7 @@ def move(m, pos, spd=5, eye=0):
     # Eyetilt
     if (motorType[m] == "Matrix Y"):
         pos = (pos - 5) * 0.4 + 5  # Bodge to reduce the scale
-        msg = "FE," + str(eye) + "," + "{:0.0f}".format(lastfex * 255 / 10) + "," + "{:0.0f}".format(
-            pos * 255 / 10) + "\n"
+        msg = "FE," + str(eye) + "," + "{:0.0f}".format(lastfex * 255 / 10) + "," + "{:0.0f}".format(pos * 255 / 10) + "\n"
         # print ("fey:" + msg)
         # Write message to serial port
         _serwrite(msg)
@@ -606,6 +645,12 @@ def _getPos(m, pos):
     return scaledPos + motorMins[m]
 
 
+# Function to set the language used by gTTS web speech synthesizer
+# name - run 'say -v ?' in terminal to find available names.
+def setLanguage(params=language):
+    global language
+    language = params
+
 # Function to set the voice used by the synthesiser
 # name - run 'say -v ?' in terminal to find available names.
 # speed - speech rate in words per min.
@@ -614,156 +659,44 @@ def setVoice(params=voice):
     global voice
     voice = params
 
-
 # Function to set a different speech synthesizer - defaults to sapi
 def setSynthesizer(params=synthesizer):
     global synthesizer
     synthesizer = params
 
-
 # Set the speed of the speech in words per min.
+def speechSpeed(params=speechRate):
+    global speechRate
+    speechRate = params
+    
+# Set the speed of the speech in words per min mirrors speechSpeed for consistency.
 def setSpeechSpeed(params=speechRate):
     global speechRate
     speechRate = params
 
-
 # Function to make Picoh Speak. Arguments | text String "Hello World" **eg say("Hello my name is Picoh")
 # untilDone - wait in function until speech is complete, lipSync - move lips in time with speech, hdmiAudio - adds a delay to give hdmi channel time to activate.
 # soundDelay - positive if lip movement is lagging behind sound, negative if sound is lagging behind lip movement.
 def say(text, untilDone=True, lipSync=True, hdmiAudio=False, soundDelay=0):
     global topLipFree
 
-    if text.isspace() or text == '':
-        return
-
-    # Use the bottom lip to push the top lip above the centre point if it is below
-
     if topLipFree:
         move(BOTTOMLIP, 4)
         wait(0.25)
-
+    
+    if text.isspace() or text == '':
+        return
     text = text.replace("picoh", "peek oh")
     text = text.replace("Picoh", "peek oh")
-
-    if platform.system() == "Linux":
-        _sayLinux(text, untilDone, lipSync, hdmiAudio, soundDelay)
-        return
-
+    
     if hdmiAudio:
         soundDelay = soundDelay - 1
 
     # Create a bash command with the desired text. espeak.exe or another synthesizer must be in the current folder.  the -w parameter forces the speech to a file
-    _speak(text)
-
-    # open the file to calculate visemes. Festival on RPi has this built in but for espeak need to do it manually
-
-    dir = os.path.dirname(os.path.abspath(__file__))
-    file = speechAudioFile
-
-    waveFile = wave.open(file, 'r')
-
-    length = waveFile.getnframes()
-    framerate = waveFile.getframerate()
-    channels = waveFile.getnchannels()
-    bytespersample = waveFile.getsampwidth()
-
-    # How many samples per second for mouth position
-    if platform.system() == "Windows":
-        VISEMESPERSEC = 10
-    if platform.system() == "Darwin":
-        VISEMESPERSEC = 10
-    if platform.system() == "Linux":
-        VISEMESPERSEC = 10
-
-    # How many samples in 1/20th second
-    # print ('framerate:', framerate, ' channels:', channels, ' length:', length, ' bytespersample:', bytespersample)
-
-    chunk = int(waveFile.getframerate() / VISEMESPERSEC)
-    # print ('chunk:', chunk)
-
-    # Empty the lists that contain phoneme data and reset count
-    phonemes = []
-    times = []
-
-    ms = 0
-
-    for i in range(0, length - chunk, chunk):
-        vol = 0
-        buffer = waveFile.readframes(chunk)
-        # frame is 1 sample for mono or 2 for stereo
-        bytesread = chunk * channels * bytespersample
-        # print ('bytesread:', bytesread)
-        index = 0;
-        for sample in range(0, int(bytesread / (channels * bytespersample))):
-            vol += buffer[index]
-            vol += buffer[index + 1] * 256
-            index += bytespersample
-            if channels > 1:
-                vol += buffer[index]
-                vol += buffer[index + 1] * 256
-                index += bytespersample
-
-        # print ('viseme', i, ":", ms, ':', vol)
-        ms += (1000 / VISEMESPERSEC);
-
-        phonemes.append(float(vol))
-        times.append(float(ms) / 1000)
-
-    # Back to the beginning for next use
-    waveFile.rewind()
-
-    # Normalise the volume
-    max = 0
-    for i in range(0, len(phonemes) - 1):
-        if phonemes[i] > max:
-            max = phonemes[i]
-
-    for i in range(0, len(phonemes) - 1):
-        phonemes[i] = phonemes[i] * 10 / max
-        # print ('visnorm', i, ":", times[i], ':', phonemes[i])
-
-    if lipSync:
-        if soundDelay > 0:
-            # Set up a thread for the speech sound synthesis, delay start by soundDelay
-            t = threading.Timer(soundDelay, _saySpeech, args=(hdmiAudio,), kwargs=None)
-            # Set up a thread for the speech movement
-            t2 = threading.Thread(target=_moveSpeech, args=(phonemes, times))
-        else:
-            # Set up a thread for the speech sound synthesis
-            t = threading.Thread(target=_saySpeech, args=(hdmiAudio,))
-            # Set up a thread for the speech movement, delay start by - soundDelay
-            t2 = threading.Timer(-soundDelay, _moveSpeech, args=(phonemes, times), kwargs=None)
-        t2.start()
-    else:
-        # Set up a thread for the speech sound synthesis
-        t = threading.Thread(target=_saySpeech, args=(hdmiAudio,))
-    t.start()
-
-    # if untilDone, keep running until speech has finished
-    if untilDone:
-        totalTime = times[len(times) - 1]
-        startTime = time.time()
-        while time.time() - startTime < totalTime:
-            continue
-
-
-def _sayLinux(text, untilDone=True, lipSync=True, hdmiAudio=False, soundDelay=0):
-    if "festival" in synthesizer.lower():
-
-        if hdmiAudio:
-            soundDelay = soundDelay - 1
-
-        safetext = re.sub(r'[^ .a-zA-Z0-9?\']+', '', text)
-
-        # Create a bash command with the desired text. The command writes two files, a .wav with the speech audio and a .txt file containing the phonemes and the times.
-
-        bashcommand = "festival -b '(set! mytext (Utterance Text " + '"' + safetext + '"))' + "' '(utt.synth mytext)' '(utt.save.wave mytext " + '"picohData/picohspeech.wav")' + "' '(utt.save.segs mytext " + '"picohData/phonemes"' + ")'"
-
-        # Execute bash command.
-        subprocess.call(bashcommand, shell=True)
-        if debug:
-            print (bashcommand)
-
+    _generateSpeechFile(text)
+    
+    if synthesizer.upper() == "FESTIVAL":
+        
         # Open the text file containing the phonemes
 
         f = open("picohData/phonemes", 'r')
@@ -789,41 +722,11 @@ def _sayLinux(text, untilDone=True, lipSync=True, hdmiAudio=False, soundDelay=0)
             if vals:
                 phonemes.append(vals[2])
                 times.append(float(vals[0]))
-
-        if lipSync:
-            if soundDelay > 0:
-                # Set up a thread for the speech sound synthesis, delay start by soundDelay
-                t = threading.Timer(soundDelay, _saySpeech, args=(hdmiAudio,), kwargs=None)
-                # Set up a thread for the speech movement
-                t2 = threading.Thread(target=_moveSpeechFest, args=(phonemes, times))
-            else:
-                # Set up a thread for the speech sound synthesis
-                t = threading.Thread(target=_saySpeech, args=(hdmiAudio,))
-                # Set up a thread for the speech movement, delay start by - soundDelay
-                t2 = threading.Timer(-soundDelay, _moveSpeechFest, args=(phonemes, times), kwargs=None)
-            t2.start()
-        else:
-            # Set up a thread for the speech sound synthesis
-            t = threading.Thread(target=_saySpeech, args=(hdmiAudio,))
-        t.start()
-
-        # if untilDone, keep running until speech has finished
-        if untilDone:
-            totalTime = times[len(times) - 1]
-            startTime = time.time()
-            while time.time() - startTime < totalTime:
-                continue
-
-    if ("espeak" in synthesizer.lower() or "pico2wave" in synthesizer.lower() or "gtts" in synthesizer.lower()):
-
-        if hdmiAudio:
-            soundDelay = soundDelay - 1
-
-        # Create a bash command with the desired text. espeak.exe or another synthesizer must be in the current folder.  the -w parameter forces the speech to a file
-        _speak(text)
-
+    else:
+        
         # open the file to calculate visemes. Festival on RPi has this built in but for espeak need to do it manually
-        waveFile = wave.open("picohData/picohspeech.wav", 'r')
+
+        waveFile = wave.open(speechAudioFile, 'r')
 
         length = waveFile.getnframes()
         framerate = waveFile.getframerate()
@@ -831,7 +734,12 @@ def _sayLinux(text, untilDone=True, lipSync=True, hdmiAudio=False, soundDelay=0)
         bytespersample = waveFile.getsampwidth()
 
         # How many samples per second for mouth position
-        VISEMESPERSEC = 20
+        if platform.system() == "Windows":
+            VISEMESPERSEC = 10
+        if platform.system() == "Darwin":
+            VISEMESPERSEC = 10
+        if platform.system() == "Linux":
+            VISEMESPERSEC = 10
 
         # How many samples in 1/20th second
         # print ('framerate:', framerate, ' channels:', channels, ' length:', length, ' bytespersample:', bytespersample)
@@ -873,52 +781,42 @@ def _sayLinux(text, untilDone=True, lipSync=True, hdmiAudio=False, soundDelay=0)
         # Normalise the volume
         max = 0
         for i in range(0, len(phonemes) - 1):
-            if (phonemes[i] > max):
+            if phonemes[i] > max:
                 max = phonemes[i]
 
         for i in range(0, len(phonemes) - 1):
             phonemes[i] = phonemes[i] * 10 / max
             # print ('visnorm', i, ":", times[i], ':', phonemes[i])
-
-        if lipSync:
-            if soundDelay > 0:
-                # Set up a thread for the speech sound synthesis, delay start by soundDelay
-                t = threading.Timer(soundDelay, _saySpeech, args=(hdmiAudio,), kwargs=None)
-                # Set up a thread for the speech movement
-                t2 = threading.Thread(target=_moveSpeech, args=(phonemes, times))
-            else:
-                # Set up a thread for the speech sound synthesis
-                t = threading.Thread(target=_saySpeech, args=(hdmiAudio,))
-                # Set up a thread for the speech movement, delay start by - soundDelay
-                t2 = threading.Timer(-soundDelay, _moveSpeech, args=(phonemes, times), kwargs=None)
-            t2.start()
+    
+    # Use the bottom lip to push the top lip above the centre point if it is below
+       
+    if lipSync:
+        if soundDelay > 0:
+            # Set up a thread for the speech sound synthesis, delay start by soundDelay
+            t = threading.Timer(soundDelay, _playSpeech, args=(hdmiAudio,), kwargs=None)
+            # Set up a thread for the speech movement
+            t2 = threading.Thread(target=_moveSpeech, args=(phonemes, times))
         else:
             # Set up a thread for the speech sound synthesis
-            t = threading.Thread(target=_saySpeech, args=(hdmiAudio,))
-        t.start()
-
-        # if untilDone, keep running until speech has finished
-        if untilDone:
-            totalTime = times[len(times) - 1]
-            startTime = time.time()
-            while time.time() - startTime < totalTime:
-                continue
-
-
-# Function to limit values so they are between 0 - 10
-def _limit(val):
-    if val > 10:
-        return 10
-    elif val < 0:
-        return 0
+            t = threading.Thread(target=_playSpeech, args=(hdmiAudio,))
+            # Set up a thread for the speech movement, delay start by - soundDelay
+            t2 = threading.Timer(-soundDelay, _moveSpeech, args=(phonemes, times), kwargs=None)
+        t2.start()
     else:
-        return val
-
+        # Set up a thread for the speech sound synthesis
+        t = threading.Thread(target=_playSpeech, args=(hdmiAudio,))
+    t.start()
 
+    # if untilDone, keep running until speech has finished
+    if untilDone:
+        totalTime = times[len(times) - 1]
+        startTime = time.time()
+        while time.time() - startTime < totalTime:
+            continue
+                
 # Function to play back the speech wav file, if hmdi audio is being used play silence before speech sound
-def _saySpeech(addSilence):
+def _playSpeech(addSilence):
     dir = os.path.dirname(os.path.abspath(__file__))
-    # speechFile = os.path.join(dir, 'picohspeech.wav')
     silenceFile = os.path.join(dir, 'Silence1.wav')
 
     speechFile = speechAudioFile
@@ -951,34 +849,28 @@ def _moveSpeech(phonemes, times):
         timeNow = time.time() - startTime
         for x in range(0, len(times)):
             if timeNow > times[x] and x > currentX:
-                posTop = _phonememapTop(phonemes[x])
-                posBottom = _phonememapBottom(phonemes[x])
+                if synthesizer.upper() == "FESTIVAL":
+                    #posTop = _phonememapTopFest(phonemes[x])
+                    posBottom = _phonememapBottomFest(phonemes[x])
+                else:
+                    posTop = _phonememapTop(phonemes[x])
+                    posBottom = _phonememapBottom(phonemes[x])
                 # move(TOPLIP, posTop, 10)
+                posBottom = (((posBottom-5)/5)*3)+5
                 move(BOTTOMLIP, posBottom, 10)
                 currentX = x
     # move(TOPLIP, 5)
     move(BOTTOMLIP, 5)
+    
 
-
-def _moveSpeechFest(phonemes, times):
-    startTime = time.time()
-    timeNow = 0
-    totalTime = times[len(times) - 1]
-    currentX = -1
-    while timeNow < totalTime:
-        timeNow = time.time() - startTime
-        for x in range(0, len(times)):
-            if timeNow > times[x] and x > currentX:
-                posTop = _phonememapTopFest(phonemes[x])
-                posBottom = _phonememapBottomFest(phonemes[x])
-                posBottom = 5 + (posBottom * 3 / 10)
-                # move(TOPLIP,posTop,10)
-
-                move(BOTTOMLIP, posBottom, 10)
-                currentX = x
-    # move(TOPLIP,5)
-    move(BOTTOMLIP, 5)
-
+# Function to limit values so they are between 0 - 10    
+def _limit(val):
+    if val > 10:
+        return 10
+    elif val < 0:
+        return 0
+    else:
+        return val
 
 # Function mapping phonemes to top lip positions. Argument | val → phoneme | returns a position as int
 def _phonememapTopFest(val):
@@ -1073,18 +965,15 @@ def _phonememapBottomFest(val):
         'v': 5
     }.get(val, 5)
 
-
 # Function mapping phonemes to top lip positions.
 def _phonememapTop(val):
     return 5 + (_limit(val) / 2)
 
-
 # Function mapping phonemes to top lip positions.
 # Bottom lip never goes over 9
 def _phonememapBottom(val):
     return 5 + (_limit(val) * 3 / 10)
 
-
 # Legacy function to support Ohbot programs with eyeColour. Passes onto baseColour.
 def eyeColour(r, g, b, swapRandG=False):
     baseColour(r, g, b, swapRandG)
@@ -1124,25 +1013,24 @@ def baseColour(r, g, b, swapRandG=False):
     _serwrite(msg1)
     _serwrite(msg2)
 
-
+# Wait for a number of seconds. 
 def wait(seconds):
     time.sleep(float(seconds))
     return
 
-
+# Detach each of Picoh's motors.
 def close():
     for x in range(0, len(motorMins) - 1):
         detach(x)
 
     # Reset Picoh back to start position
-
-
 def reset():
     baseColour(0, 0, 0)
-    setEyeShape("Eyeball", "Eyeball")
-    for x in range(0, len(restPos) - 1):
+    setEyeShape(defaultEyeShape, defaultEyeShape)
+    for x in range(len(restPos) - 1,-1,-1):
         move(x, restPos[x])
-
+        wait(0.1)
+    close()
 
 # Return the sensor value between 0-10 for a given sensor number. Values stored in sensors[] array.
 def readSensor(index):
@@ -1178,7 +1066,6 @@ def setEyeBrightness(val):
 
     _serwrite("FI," + msg + "\n")
 
-
 # set the eye shape according to the passed in eyeshapedefinition
 # eysshape definition is 6 sets of 9 hex pairs which set the bits of half of the screen
 # the other half of the screen is a mirror copy
@@ -1205,7 +1092,6 @@ def _setEyes(leftDefinition, rightDefinition="", autoMirror=True):
     # Pupil is held in set 6 and has been implemented in the Arduino driver as FB 8
     _serwrite("FB,8," + _EyeShapeBytes(definition, rightDefinition, 5, autoMirror) + "\n")
 
-
 # function for getting a string that defines the 16 x 9 matrix for a particular 8 x 9 eyeshapedefinition
 # setNo is 0 for the normal eyeshape, 1 to 4 for blink shapes or 5 for the pupil
 def _EyeShapeBytes(definitionR, definition, setNo, autoMirror):
@@ -1221,12 +1107,11 @@ def _EyeShapeBytes(definitionR, definition, setNo, autoMirror):
             strRet += _reverseBits(definition[offset: offset + 2])
 
         strRet += _reverseBits(definitionR[offset: offset + 2])
-
-    # print ("eyeshape set:" + str(set) + ": " + strRet)
+    if debug:
+        print ("eyeshape set:" + str(set) + ": " + strRet)
 
     return strRet
 
-
 # reverse the bits of a two byte hex number
 def _reverseBits(str):
     # print ("str: " + str)
@@ -1253,7 +1138,6 @@ def _reverseBits(str):
     # https://stackoverflow.com/questions/2269827/how-to-convert-an-int-to-a-hex-string
     return "%0.2X" % r
 
-
 def setEyeShape(shapeNameRight, shapeNameLeft=''):
     global shapeList
 
@@ -1303,7 +1187,6 @@ def getPhrase(set='None', variable='None'):
     else:
         return possiblePhrases[random.randint(0, length - 1)]
 
-
 # Function to play sounds located in the picohData/Sounds/ folder.
 def playSound(name="", untilDone=True):
     if not name:
@@ -1319,14 +1202,12 @@ def playSound(name="", untilDone=True):
     if untilDone:
         soundThread.join()
 
-
 def _playSoundThread(name=""):
     if not name:
         return
 
     name = name + ".wav"
-    dir = soundFolder
-    soundFile = os.path.join(dir, name)
+    soundFile = os.path.join(soundFolder, name)
 
     # play the sound on Windows
     if platform.system() == "Windows":
@@ -1338,4 +1219,4 @@ def _playSoundThread(name=""):
 
     # play the sound on Linux
     if platform.system() == "Linux":
-        os.system('aplay ' + soundFile)
\ No newline at end of file
+        os.system('aplay ' + soundFile)
